// -*- mode:C++ ; compile-command: "/usr/bin/g++ -g app_sig.cc sha256.c -Wall -o app_sig -DDFU_PROTECT=0x90700000" -*-
// Pour pouvoir certifier compiler puis faire depuis le repertoire build
// ./app_sig KhiCAS HexEdit Nofrendo Peanut-GB
// recopier apps_sha256.h dans le repertoire ~/khi18/apps/external et ~/khi2/apps/external
// pour certifier le bootloader faire
// ./app_sig bootloader.bin 
// puis copier le tableau de apps_sha256.h dans kdisplay.cc BYTE bootloader_hash1[]=...
#include <sys/stat.h>
#include <sys/types.h>
#include "sha256.h"
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>
using namespace std;
const char sigfname[]="apps_sha256.h";

std::string remove_path(const std::string & st){
  int s=int(st.size()),i;
  for (i=s-1;i>=0;--i){
    if (st[i]=='/')
	break;
  }
  string res=st.substr(i+1,s-i-1);
  if (res=="certif.tar")
    res="apps.tar";
  if (res=="khi.A.bin")
    res="delta.external.bin";
  return res;
}

int main(int argc,const char ** argv){
#ifdef DFU_PROTECT
  unsigned i=DFU_PROTECT;
  printf("With this DFU_PROTECT flag, protection stops at 0x%x\n",i);
#else
  printf("To protect external flash, compile with -DDFU_PROTECT=address\n");  
#endif
  if (argc<2)
    return 0;
  ofstream of(sigfname);
  // BYTE hash[MAXKEYS][SHA256_BLOCK_SIZE];
  BYTE buf[SHA256_BLOCK_SIZE];
  SHA256_CTX ctx;
  of << "sha256_t shakeys[]={\n";
  for (int j=1;j<argc;++j){
    bool bl=strcmp(argv[j],"bootloader.bin")==0;
    size_t taille=0;
    string text;
    FILE * f=fopen(argv[j],"r");
    if (!f)
      return 0;
    for (;;++taille){
      unsigned char c=fgetc(f);
      if (feof(f))
	break;
      text += c;
    }
    fclose(f);
    unsigned char * ptr=(unsigned char *)text.c_str();
    // decrease taille if it ends up with 0, that way we can certify tar archive
    // that begins the same as our certified tar
    // (update a certified tar with tar uvf ...)
    taille=text.size();
    // cerr << "begin " << 1e-6*clock() << "\n";
    // while (taille>4 && ptr[taille-1]==0 && ptr[taille-2]==0 && ptr[taille-3]==0 && ptr[taille-4]==0) taille-=4;
    sha256_init(&ctx);
    if (bl){
      sha256_update(&ctx, ptr, 16*1024);
      sha256_update(&ctx, ptr+32*1024, taille-32*1024);
    }
    else
      sha256_update(&ctx, ptr, taille);
    sha256_final(&ctx, buf);
    // cerr << "end " << 1e-6*clock() << "\n";
    of << '{' <<  taille << ", {";
    for (int i=0;i<SHA256_BLOCK_SIZE;++i){
      of << (unsigned) buf[i] << ",";
      // bcd convert
      //int b=buf[i];
      //b = (b/100+'0')*65536+(((b/10)%10)+'0')*256+((b%10)+'0');
    }
    of << "}},\n";
  }
  of << "};\n";
  of.close();
  cout << "File " << sigfname << " generated, should be copied in apps/external and archive.cpp in external should be regenerated\n";
  return 0;
}
